def get_callable(kls):
    """
    Converts a string to a callable object.
    Courtesy: http://stackoverflow.com/questions/452969/does-python-have-an-equivalent-to-java-class-forname/452981#452981
    """
    parts = kls.split('.')
    module = ".".join(parts[:-1])
    m = __import__(module)
    for comp in parts[1:]:
        m = getattr(m, comp)
    return m



from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import Q
from django.db.models.query import QuerySet


class AlertQuerySet(QuerySet):
    def for_recipient(self, recipient):
        return self.filter(recipient=recipient)

    def by_tag(self, tag):
        return self.filter(tag=tag)

    def by_tags(self, tags):
        search_q = Q()
        for tag in tags:
            search_q |= Q(**{'tag__exact': tag})
        return self.filter(search_q)

    def of_subject(self, subject):
        return self.filter(subject_type=ContentType.objects.get_for_model(subject), subject_id=subject.pk)

    def read(self):
        return self.filter(is_read=True)

    def unread(self):
        return self.filter(is_read=False)
    
    def pushed(self):
        return self.filter(is_pushed=True)
    
    def not_pushed(self):
        return self.filter(is_pushed=False)
    
    def email_sent(self):
        return self.filter(is_email_sent=True)
    
    def not_email_sent(self):
        return self.filter(is_email_sent=False)

    def mark_as_read(self, recipient):
        """
        Mark as read any unread messages in the current queryset for specified recipient.
        """
        # We want to filter out read ones, as later we will store
        # the time they were marked as read.
        qs = self.unread().for_recipient(recipient)
        qs.update(is_read=True)

    def mark_as_unread(self, recipient):
        """
        Mark as unread any read messages in the current queryset for specified recipient.
        """
        # We want to filter out read ones, as later we will store
        # the time they were marked as read.
        qs = self.read().for_recipient(recipient)
        qs.update(is_read=False)


class AlertManager(models.Manager):
    def get_query_set(self):
        return AlertQuerySet(self.model, using=self._db)

    def for_recipient(self, user):
        return self.get_query_set().for_recipient(user)

    def by_tag(self, tag):
        return self.get_query_set().by_tag(tag)

    def of_subject(self, subject):
        return self.get_query_set().of_subject(subject)

    def read(self):
        return self.get_query_set().read()

    def unread(self):
        return self.get_query_set().unread()
    
    def pushed(self):
        return self.get_query_set().pushed()
    
    def not_pushed(self):
        return self.get_query_set().not_pushed()
    
    def email_sent(self):
        return self.get_query_set().email_sent()
    
    def not_email_sent(self):
        return self.get_query_set().not_email_sent()

    def mark_as_read(self, recipient):
        return self.get_query_set().mark_as_read(recipient)

    def mark_as_unread(self, recipient):
        return self.get_query_set().mark_as_unread(recipient)



from django.contrib import admin

from models import Alert


class AlertAdmin(admin.ModelAdmin):
    list_display = ('pk', 'level', 'tag', 'recipient', 'subject', 'is_read', 'is_email_sent', 'is_pushed', 'created')
    list_filter = ('level', 'tag')

admin.site.register(Alert, AlertAdmin)



from django.conf import settings
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.translation import ugettext_lazy as _

from managers import AlertManager
from utils import get_callable


class Alert(models.Model):
    LEVEL_INFO = 'INFO'
    LEVEL_SUCCESS = 'SUCCESS'
    LEVEL_WARNING = 'WARNING'
    LEVEL_ERROR = 'ERROR'
    LEVELS = (
        (LEVEL_INFO, _('info')),
        (LEVEL_SUCCESS, _('success')),
        (LEVEL_WARNING, _('warning')),
        (LEVEL_ERROR, _('error'))
    )
    recipient = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_(u'recipient'), related_name='alerts')
    tag = models.CharField(verbose_name=_(u'tag'), max_length=128, choices=getattr(settings, 'ALERTS_TAGS', None))
    level = models.CharField(verbose_name=_(u'level'), max_length=16, choices=LEVELS, default=LEVEL_INFO)
    subject = GenericForeignKey('subject_type', 'subject_id')
    subject_id = models.PositiveIntegerField()
    subject_type = models.ForeignKey(ContentType, related_name='alert_subject')
    is_read = models.BooleanField(_(u'read'), default=False)
    is_email_sent = models.BooleanField(_(u'email sent'), default=False)
    is_pushed = models.BooleanField(_(u'pushed'), default=False)
    created = models.DateTimeField(_(u'created'), auto_now_add=True)
    modified = models.DateTimeField(_(u'modified'), auto_now=True)
    objects = AlertManager()

    class Meta:
        db_table = 'alerts_alert'
        verbose_name = _(u'alert')
        verbose_name_plural = _(u'alerts')
        ordering = ('-created', )

    def __unicode__(self):
        return self.message

    @property
    def message(self):
        message_handler = getattr(settings, 'ALERTS_MESSAGE_HANDLER', None)
        if message_handler is None:
            raise ImproperlyConfigured('ALERTS_MESSAGE_HANDLER is not set')
        return unicode(get_callable(message_handler)(self))

    def mark_as_read(self):
        if not self.is_read:
            self.is_read = True
            self.save()

    def mark_as_unread(self):
        if self.is_read:
            self.is_read = False
            self.save()



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Alert'
        db.create_table('alerts_alert', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('recipient', self.gf('django.db.models.fields.related.ForeignKey')(related_name='alerts', to=orm['accounts.User'])),
            ('tag', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('level', self.gf('django.db.models.fields.CharField')(default='INFO', max_length=16)),
            ('subject_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('subject_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='alert_subject', to=orm['contenttypes.ContentType'])),
            ('is_read', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_email_sent', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_pushed', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal(u'alerts', ['Alert'])


    def backwards(self, orm):
        # Deleting model 'Alert'
        db.delete_table('alerts_alert')


    models = {
        'accounts.user': {
            'Meta': {'ordering': "('first_name', 'last_name')", 'object_name': 'User', 'db_table': "'megalist_users'"},
            'address_line_1': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'address_line_2': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'country': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': u"orm['cities.Country']", 'null': 'True', 'blank': 'True'}),
            'cover': ('django.db.models.fields.files.ImageField', [], {'default': 'None', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_of_birth': ('django.db.models.fields.DateField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '75'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_admin': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_email_notifications_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_push_notifications_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'en'", 'max_length': '2'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'middle_name': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'phone': ('megalist.thirdparty.phonenumber_field.modelfields.PhoneNumberField', [], {'default': 'None', 'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'photo': ('django.db.models.fields.files.ImageField', [], {'default': 'None', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'postcode': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'postfix': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'prefix': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'timezone': ('timezone_field.fields.TimeZoneField', [], {'default': "'UTC'"}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"})
        },
        u'alerts.alert': {
            'Meta': {'ordering': "('-created',)", 'object_name': 'Alert'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_email_sent': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_pushed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_read': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'level': ('django.db.models.fields.CharField', [], {'default': "'INFO'", 'max_length': '16'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'recipient': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'alerts'", 'to': "orm['accounts.User']"}),
            'subject_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'subject_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'alert_subject'", 'to': u"orm['contenttypes.ContentType']"}),
            'tag': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'cities.country': {
            'Meta': {'ordering': "['name']", 'object_name': 'Country'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '2', 'db_index': 'True'}),
            'continent': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'db_index': 'True'}),
            'population': ('django.db.models.fields.IntegerField', [], {}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'tld': ('django.db.models.fields.CharField', [], {'max_length': '5'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['alerts']


from django import template
from django.utils.safestring import mark_safe

from ..models import Alert

register = template.Library()


@register.simple_tag(takes_context=True)
def alerts(context, tags, subject=None):
    """
    Retrieves number of unread alerts by ``tag`` for current user from request.
    Notifications could be specified by optional argument ``subject`` as well,

    Syntax::

        {% notifications 'tag' ['subject'] %}

    Example usage::

        {% notifications 'new-comments' %}             ... shows number of all unread comments of user articles
        {% notifications 'new-comments' article %}     ... shows number all unread comments of specified article
    """

    user = context['request'].user

    tags = tags.split('|')
    alerts = Alert.objects.unread().for_recipient(user).by_tags(tags)
    if subject:
        alerts = alerts.of_subject(subject)
    number = alerts.count()

    if number > 0:
        return mark_safe('<span class="badge badge-notification">%d</span>' % number)
    return ''
